var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __classPrivateFieldSet = (this && this.__classPrivateFieldSet) || function (receiver, privateMap, value) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to set private field on non-instance");
    }
    privateMap.set(receiver, value);
    return value;
};
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, privateMap) {
    if (!privateMap.has(receiver)) {
        throw new TypeError("attempted to get private field on non-instance");
    }
    return privateMap.get(receiver);
};
var _applicationID, _publicKey, _token, _commands, _port;
import dotenv from 'dotenv';
import fastify from 'fastify';
import fetch from 'node-fetch';
import rawBody from 'fastify-raw-body';
import { InteractionType, InteractionCallbackType, } from './api/api';
import { validateRequest } from './api/validate';
import Interaction from './interaction';
dotenv.config();
export default class Application {
    constructor({ applicationID, publicKey, token, port }) {
        _applicationID.set(this, void 0);
        _publicKey.set(this, void 0);
        _token.set(this, void 0);
        _commands.set(this, void 0);
        _port.set(this, void 0);
        if (applicationID == null) {
            throw new Error('Please provide an Application ID. You can find this value <here>');
        }
        if (publicKey == null) {
            throw new Error('Please provide a Public Key. You can find this value <here>');
        }
        if (token == null) {
            throw new Error('Please provide a Token. You can find this value <here>');
        }
        __classPrivateFieldSet(this, _applicationID, applicationID);
        __classPrivateFieldSet(this, _publicKey, publicKey);
        __classPrivateFieldSet(this, _token, token);
        __classPrivateFieldSet(this, _commands, new Map());
        __classPrivateFieldSet(this, _port, port !== null && port !== void 0 ? port : 3000);
    }
    addCommand(command) {
        if (__classPrivateFieldGet(this, _commands).has(command.name.toLowerCase())) {
            throw new Error(`Error registering ${command.name.toLowerCase()}: Duplicate names are not allowed`);
        }
        console.log(`Registering the ${command.name.toLowerCase()} command`);
        __classPrivateFieldGet(this, _commands).set(command.name.toLowerCase(), command);
        return this;
    }
    addCommands(...commands) {
        commands.forEach(command => this.addCommand(command));
        return this;
    }
    // TODO: Should this be moved into Command?
    updateCommands() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('Updating Commands in Development Server');
            // TODO: Move this into an API module
            const request = yield fetch(`https://discord.com/api/v8/applications/${__classPrivateFieldGet(this, _applicationID)}/guilds/${process.env.DEVELOPMENT_SERVER_ID}/commands`, {
                headers: {
                    'Content-Type': 'application/json',
                    Authorization: `Bot ${__classPrivateFieldGet(this, _token)}`,
                    'User-Agent': 'InteractionKit (https://interactionkit.dev, 0.0.1)'
                },
            });
            const json = yield request.json();
            // TODO: Handle errors
            /**
             * Not in development server:
             *  { message: 'Missing Access', code: 50001 }
             */
            __classPrivateFieldGet(this, _commands).forEach((command) => __awaiter(this, void 0, void 0, function* () {
                const signature = json.find(cmd => cmd.name === command.name);
                if (signature == null) {
                    console.log(`\tCreating ${command.name}`);
                    const createResponse = yield fetch(`https://discord.com/api/v8/applications/${__classPrivateFieldGet(this, _applicationID)}/guilds/${process.env.DEVELOPMENT_SERVER_ID}/commands`, {
                        headers: {
                            'Content-Type': 'application/json',
                            Authorization: `Bot ${__classPrivateFieldGet(this, _token)}`,
                            'User-Agent': 'InteractionKit (https://interactionkit.dev, 0.0.1)'
                        },
                        method: 'POST',
                        body: JSON.stringify(command.toJSON()),
                    });
                    if (!createResponse.ok) {
                        console.error(`\tProblem updating ${command.name}`);
                        const createJSON = yield createResponse.json();
                        console.error(createJSON);
                    }
                }
                else if (!command.isEqualTo(signature)) {
                    console.log(`\tUpdating ${command.name}`);
                    const updateResponse = yield fetch(`https://discord.com/api/v8/applications/${__classPrivateFieldGet(this, _applicationID)}/guilds/${process.env.DEVELOPMENT_SERVER_ID}/commands/${signature.id}`, {
                        headers: {
                            'Content-Type': 'application/json',
                            Authorization: `Bot ${__classPrivateFieldGet(this, _token)}`,
                            'User-Agent': 'InteractionKit (https://interactionkit.dev, 0.0.1)'
                        },
                        method: 'PUT',
                        body: command.toJSON(),
                    });
                    if (!updateResponse.ok) {
                        console.error(`Problem updating ${command.name}`);
                    }
                }
            }));
            return this;
        });
    }
    // loadDirectory(path: string) {
    // TODO: Load all JS files from path
    // TODO: Create map of file/commandData
    // TODO: Create file listener on change
    // TODO: onChange, reload file and maybe emit command change events
    // }
    startServer(callback) {
        console.log('Starting server...');
        const server = fastify();
        server.register(rawBody, {
            runFirst: true,
        });
        server.addHook('preHandler', (request, response) => __awaiter(this, void 0, void 0, function* () {
            if (request.method === 'POST') {
                if (!validateRequest(request, __classPrivateFieldGet(this, _publicKey))) {
                    console.log('Invalid Discord Request');
                    return response.status(401).send({ error: 'Bad request signature ' });
                }
            }
        }));
        server.post('/', (request, response) => __awaiter(this, void 0, void 0, function* () {
            var _a;
            console.log('REQUEST');
            const interaction = new Interaction(request, response);
            if (interaction == null || interaction.type === InteractionType.PING) {
                console.log('Handling Discord Ping');
                response.send({
                    type: InteractionCallbackType.PONG,
                });
            }
            else if (interaction.type === InteractionType.APPLICATION_COMMAND) {
                console.log({ interaction });
                if (__classPrivateFieldGet(this, _commands).has(interaction.name)) {
                    console.log(`Handling ${interaction.name}`);
                    return (_a = __classPrivateFieldGet(this, _commands).get(interaction.name)) === null || _a === void 0 ? void 0 : _a.handler(interaction, this);
                }
                else {
                    console.error(`Unknown Command: ${interaction.name}`);
                    response.status(400).send({ error: 'Unknown Type' });
                }
            }
            else {
                // TODO: figure out what would lead to this state, and how to handle it.
                console.error(`Unknown Type: ${interaction.type}`);
            }
        }));
        if (callback != null) {
            callback(server);
        }
        else {
            server.listen(__classPrivateFieldGet(this, _port), (error, address) => __awaiter(this, void 0, void 0, function* () {
                if (error) {
                    console.error(error);
                    process.exit(1);
                }
                console.log(`Server listening on ${address}`);
            }));
        }
        // TODO: Move this into a dev env check.
        this.updateCommands();
    }
}
_applicationID = new WeakMap(), _publicKey = new WeakMap(), _token = new WeakMap(), _commands = new WeakMap(), _port = new WeakMap();
//# sourceMappingURL=application.js.map